//+------------------------------------------------------------------+
//|                                                  AlexanderEA.mq5 |
//+------------------------------------------------------------------+
/******************************************
Ein ganz einfacher EA in MQL5 - MT5
Spread einstellbar 0.00
Kaufen wenn der Tick - Ping grösser als 0.02 pip - Kaufen/Verkaufen
Tailingstop 0.01 pip
ganz einfach ohne Seitenlange Berechnungen, das einzige ist zu warten bis ein Tick reinkommt 
der grösser als 0.02 Pip ist. Den EA so dass ich ihn selbst noch bearbeiten kann. 
Dieser EA ist nur ein Teil des ganzen.
*****************************************/
#property version   "1.0"

#include <Trade/Trade.mqh>
#include <Trade/SymbolInfo.mqh>
#include <Trade/PositionInfo.mqh>
              
CTrade         m_trade;                  
CSymbolInfo    m_symbol;  
CPositionInfo  m_position;

input double    minTick    =   0.06;  // min. Tick (z.B. 0.02 / 0 = aus)
input int    TS            =   5;     // Trailingstop (z.B. 0.01)
input double    Lot        =   0.1;      // Lot
input int    SL            =   50;        // Stoploss (Point)
input int    TP            =   85;        // Takeprofit (Point)
input int    MagicNr       = 123456;

ulong ticketNr;
double lastBid;


//+------------------------------------------------------------------+
//| OnInit()
//+------------------------------------------------------------------+
int OnInit()
{
   Comment("START");
   
   lastBid = m_symbol.Bid();
   m_symbol.Name(Symbol());  // set symbol name
   m_trade.SetExpertMagicNumber(MagicNr);
   
   return(INIT_SUCCEEDED);
}


//+------------------------------------------------------------------+
//| Beispiel: Setze Stop-Loss oberhalb des Kaufpreises               |
//+------------------------------------------------------------------+
void SetStopLoss(double buyPrice)
{
   // Definiere den Stop-Loss Abstand
   double stopLossOffset = 10.05;

   // Berechne den Stop-Loss-Wert
   double stopLoss = buyPrice + stopLossOffset;

   // Setze den Stop-Loss für die aktuelle Position
   if(PositionSelect(Symbol())) // Überprüfe, ob eine Position aktiv ist
   {
      MqlTradeRequest request;
      MqlTradeResult result;
      ZeroMemory(request);
      ZeroMemory(result);

      request.action = TRADE_ACTION_SLTP;
      request.symbol = Symbol();
      request.sl = stopLoss;

      if(!OrderSend(request, result))
      {
         Print("Fehler beim Setzen des Stop-Loss: ", GetLastError());
      }
      else
      {
         Print("Stop-Loss erfolgreich gesetzt auf: ", stopLoss);
      }
   }
   else
   {
      Print("Keine Position offen, Stop-Loss kann nicht gesetzt werden.");
   }
}


//+------------------------------------------------------------------+
void OnTick()
{
   //+------------------------------------------------------------------  Profit?
   calculateProfit();
   Comment("Profit: " + DoubleToString(historyProfit, 2), " | Ask (= Kauf) - Bid (= Verkauf) = ", 
   NormalizeDouble(m_symbol.Ask() - m_symbol.Bid(), _Digits));
   
  
   //+------------------------------------------------------------------ Position vorhanden?
   
   bool hasPosition = false;
   
   for (int i = PositionsTotal()-1; i>=0; i--) {
      ticketNr = PositionGetTicket(i); // Position laden
      
      if (m_position.SelectByTicket(ticketNr)) {  // Position laden
         if (m_position.Symbol() != m_symbol.Name()) continue; // anderes Instrument?
         if (m_position.Magic() != MagicNr) continue; // anderer EA?
         
         //Comment("Position existiert! Ticket = ", ticketNr);
         hasPosition = true;
         break;
      }
   }
   
   //+------------------------------------------------------------------ Trailing
   
   if (hasPosition) {
      trailPositions();
      lastBid = m_symbol.Bid(); // letzten Preis merken
      
      //Comment("Wegen Position nicht mehr kaufen");
      return; // nicht mehr kaufen
   }
   
   
   //+------------------------------------------------------------------
   //+------------------------------------------------------------------ 
   //+------------------------------------------------------------------
   


   //+------------------------------------------------------------------  Kaufen/Verkaufen
   
   if (minTick <= 0) return; // Ausschalter
   
   if (lastBid != m_symbol.Bid()) { 
      Comment(m_symbol.Bid() - lastBid);
      
      if (m_symbol.Bid() >= (lastBid + minTick)) {
      
         double sl = SL > 0 ? NormalizeDouble(m_symbol.Ask() + SL * _Point,_Digits) : 0;
         double tp = TP > 0 ? NormalizeDouble(m_symbol.Ask() + TP * _Point,_Digits) : 0;
         double lot = NormalizeLots(Lot);
   
         m_trade.Buy(lot,m_symbol.Name(),m_symbol.Ask(),sl,tp,"");
      }
      else if (m_symbol.Bid() <= (lastBid - minTick)) {
      
         double sl = SL > 0 ? NormalizeDouble(m_symbol.Bid() + SL * _Point,_Digits) : 0;
         double tp = TP > 0 ? NormalizeDouble(m_symbol.Bid() - TP * _Point,_Digits) : 0;
         double lot = NormalizeLots(Lot);
   
         m_trade.Sell(lot,m_symbol.Name(),m_symbol.Ask(),sl,tp,"");
      }
   }
   lastBid = m_symbol.Bid(); // letzten Preis merken
   
}
//+------------------------------------------------------------------+


//+------------------------------------------------------------------+
// NormalizeLots
//+------------------------------------------------------------------+
double NormalizeLots(double value)
{
   double min = SymbolInfoDouble(_Symbol, SYMBOL_VOLUME_MIN);
   double max = SymbolInfoDouble(_Symbol, SYMBOL_VOLUME_MAX);
   double st = SymbolInfoDouble(_Symbol, SYMBOL_VOLUME_STEP);
   
   double rounded = st*(int)MathRound(value / st);
   
   rounded = MathMax(rounded, min);
   rounded = MathMin(rounded, max);
   rounded = NormalizeDouble(rounded, 2);
   
   return rounded;
}

//+------------------------------------------------------------------+
// trailPositions() 
//+------------------------------------------------------------------+
void trailPositions() 
{
   if (TS <= 0) return;
   
   CPositionInfo position;
   CTrade trade;
   
   double stopLoss = TS; // * _Point;
   
   for (int i = PositionsTotal()-1; i >= 0; i--) {
  
      if (position.SelectByIndex(i) && position.Symbol() == Symbol() && position.Magic() == MagicNr) {
      
         ENUM_POSITION_TYPE type = position.PositionType();
         double CurrentSL = position.StopLoss();
         double CurrentTP = position.TakeProfit();
         double CurrentPrice = position.PriceCurrent();
 
         if (type == POSITION_TYPE_BUY) {
            if (CurrentPrice - stopLoss > CurrentSL || CurrentSL == 0.0) {
               trade.PositionModify(position.Ticket(), NormalizeDouble((CurrentPrice - stopLoss), Digits()), CurrentTP);
            }
         }
           
         if (type == POSITION_TYPE_SELL) {
            if (CurrentPrice + stopLoss < CurrentSL || CurrentSL == 0.0) {
               trade.PositionModify(position.Ticket(), NormalizeDouble((CurrentPrice + stopLoss), Digits()), CurrentTP);
            }
         }
      }
   }
}

//+------------------------------------------------------------------+
// calculateProfit (History)
//+------------------------------------------------------------------+
int historyPositions = 0;
double historyProfit = 0.0;
   
void calculateProfit()
{
   // vor der Neuberechnung zurück setzen:
   historyPositions = 0;
   historyProfit = 0.0;
   
   HistorySelect(0,TimeCurrent()); 
   
   for (int i=(HistoryDealsTotal()-1); i >= 0; i--) {
      ulong ticketDeal = HistoryDealGetTicket(i);
      
      if (ticketDeal > 0) {
         if (HistoryDealGetString(ticketDeal,DEAL_SYMBOL) != Symbol()) continue;
         if (HistoryDealGetInteger(ticketDeal,DEAL_MAGIC) != MagicNr) continue;
         
         if (HistoryDealGetInteger(ticketDeal,DEAL_ENTRY) == DEAL_ENTRY_OUT) {
         
            double profit = HistoryDealGetDouble(ticketDeal,DEAL_PROFIT)
                          + HistoryDealGetDouble(ticketDeal,DEAL_SWAP)
                          + HistoryDealGetDouble(ticketDeal,DEAL_COMMISSION);
            
            historyPositions++;
            historyProfit += profit;
         }
      }
   }
} 

